=== modified file 'NEWS'
--- NEWS	2012-03-01 21:29:00 +0000
+++ NEWS	2012-07-09 15:23:26 +0000
@@ -8,6 +8,17 @@
   * Properly handle popup menus when there is no selected revision in
     "bzr viz". (Jelmer Vernooij, #281717)
 
+  * Ignore bzr-handle-patch.desktop in sofware center.
+    (Jelmer Vernooij, #854245)
+
+  * Fix searching in tree views. (Jelmer Vernooij, #928301)
+
+ CHANGES
+
+  * bzr-notify has been removed, as it was causing regressions
+    on several machines and not particularly useful.
+    (Jelmer Vernooij, #714767, #949798, #998994, #342135, #342136, #394536)
+
 0.103.0	2011-12-11
 
  FEATURES

=== modified file '__init__.py'
--- __init__.py	2011-12-19 17:47:22 +0000
+++ __init__.py	2012-03-16 17:28:20 +0000
@@ -53,10 +53,6 @@
 
 import bzrlib
 import bzrlib.api
-from bzrlib import (
-    branch,
-    config,
-    )
 from bzrlib.commands import plugin_cmds
 
 from bzrlib.plugins.gtk.info import (
@@ -134,14 +130,30 @@
     from bzrlib.plugins.gtk import commitmsgs
     commitmsgs.save_commit_messages(*args)
 
-branch.Branch.hooks.install_named_hook('post_uncommit',
-                                       save_commit_messages,
-                                       "Saving commit messages for gcommit")
-
-option_registry = getattr(config, "option_registry", None)
-if option_registry is not None:
-    config.option_registry.register_lazy('nautilus_integration',
-            'bzrlib.plugins.gtk.config', 'opt_nautilus_integration')
+try:
+    from bzrlib.hooks import install_lazy_named_hook
+except ImportError:
+    from bzrlib.branch import Branch
+    Branch.hooks.install_named_hook('post_uncommit',
+                                    save_commit_messages,
+                                    "Saving commit messages for gcommit")
+else:
+    install_lazy_named_hook("bzrlib.branch", "Branch.hooks",
+        'post_uncommit', save_commit_messages, "Saving commit messages for gcommit")
+
+try:
+    from bzrlib.registry import register_lazy
+except ImportError:
+    from bzrlib import config
+    option_registry = getattr(config, "option_registry", None)
+    if option_registry is not None:
+        config.option_registry.register_lazy('nautilus_integration',
+                'bzrlib.plugins.gtk.config', 'opt_nautilus_integration')
+else:
+    register_lazy("bzrlib.config", "option_registry",
+        'nautilus_integration', 'bzrlib.plugins.gtk.config',
+        'opt_nautilus_integration')
+
 
 def load_tests(basic_tests, module, loader):
     testmod_names = [

=== modified file 'branchview/treeview.py'
--- branchview/treeview.py	2012-03-01 21:29:00 +0000
+++ branchview/treeview.py	2012-03-23 15:02:38 +0000
@@ -354,7 +354,7 @@
         # from the key (that is the key is found in a REVNO at the offset 0)
         # or if a MESSAGE *contains* the key anywhere (that is, the key is
         # found case insensitively in a MESSAGE at any offset)
-        def search_equal_func(model, column, key, iter):
+        def search_equal_func(model, column, key, iter, ignored):
             return (model.get_value(iter, treemodel.REVNO).find(key) != 0
                 and model.get_value(iter, treemodel.MESSAGE).lower().find(key.lower()) == -1)
 

=== modified file 'bzr-handle-patch'
--- bzr-handle-patch	2012-02-03 22:55:40 +0000
+++ bzr-handle-patch	2012-03-23 12:56:39 +0000
@@ -8,34 +8,34 @@
 import sys
 
 if len(sys.argv) < 2:
-	print "Usage: handle-patch <path>"
-	sys.exit(1)
+    print "Usage: handle-patch <path>"
+    sys.exit(1)
 
 
 path = sys.argv[1]
 
 try:
-	from bzrlib.plugins.gtk.diff import (DiffController,
-										 MergeDirectiveController)
-	if path == '-':
-		lines = sys.stdin.readlines()
-	else:
-		lines = open(path, 'rb').readlines()
-	lines = [l.replace('\r\n', '\n') for l in lines]
-	try:
-		directive = merge_directive.MergeDirective.from_lines(lines)
-	except errors.NotAMergeDirective:
-		controller = DiffController(path, lines, allow_dirty=True)
-	else:
-		controller = MergeDirectiveController(path, directive)
-	window = controller.window
-	window.show()
-	Gtk = open_display()
-	window.connect("destroy", Gtk.main_quit)
+    from bzrlib.plugins.gtk.diff import (DiffController,
+                                         MergeDirectiveController)
+    if path == '-':
+        lines = sys.stdin.readlines()
+    else:
+        lines = open(path, 'rb').readlines()
+    lines = [l.replace('\r\n', '\n') for l in lines]
+    try:
+        directive = merge_directive.MergeDirective.from_lines(lines)
+    except errors.NotAMergeDirective:
+        controller = DiffController(path, lines, allow_dirty=True)
+    else:
+        controller = MergeDirectiveController(path, directive)
+    window = controller.window
+    window.show()
+    Gtk = open_display()
+    window.connect("destroy", Gtk.main_quit)
 except Exception, e:
-	from bzrlib.plugins.gtk.dialog import error_dialog
-	error_dialog('Error', str(e))
-	raise
+    from bzrlib.plugins.gtk.dialog import error_dialog
+    error_dialog('Error', str(e))
+    raise
 
 if len(sys.argv) == 3 and sys.argv[2] == 'test':
     sys.exit(0)

=== modified file 'bzr-handle-patch.desktop'
--- bzr-handle-patch.desktop	2011-03-06 08:24:07 +0000
+++ bzr-handle-patch.desktop	2012-03-23 13:45:02 +0000
@@ -8,3 +8,4 @@
 Type=Application
 Categories=Application;Development;RevisionControl;
 MimeType=text/x-diff;text/x-patch;application/x-bazaar-merge-directive;
+X-AppInstall-Ignore=true

=== removed file 'bzr-notify'
--- bzr-notify	2012-02-03 19:49:55 +0000
+++ bzr-notify	1970-01-01 00:00:00 +0000
@@ -1,102 +0,0 @@
-#!/usr/bin/env python
-
-"""Run the bzr tray icon.
-
-This is a background program which will pop up a notification on the users
-screen when a commit occurs.
-"""
-
-from bzrlib.plugin import load_plugins
-load_plugins()
-
-from bzrlib.plugins.gtk.commands import open_display
-
-from bzrlib.plugins.gtk.notify import NotifyPopupMenu
-Gtk = open_display()
-
-import cgi
-import sys
-
-import dbus
-import dbus.service
-from gi.repository import GObject
-from gi.repository import Notify
-
-from bzrlib.bzrdir import BzrDir
-from bzrlib.osutils import format_date
-from bzrlib.transport import get_transport
-
-menu = NotifyPopupMenu()
-try:
-    from gi.repository import AppIndicator3
-except ImportError:
-    icon = Gtk.StatusIcon.new_from_icon_name("bzr-panel")
-    icon.connect('popup-menu', menu.display)
-    icon.set_visible(False)
-    hide_icon = lambda: icon.set_visible(False)
-    show_icon = lambda: icon.set_visible(True)
-else:
-    indicator = AppIndicator3.Indicator.new(
-        "bzr-gtk-notify", "bzr-panel", AppIndicator3.IndicatorCategory.OTHER)
-    indicator.set_status (AppIndicator3.IndicatorStatus.PASSIVE)
-    indicator.set_attention_icon("bzr-panel")
-    indicator.set_menu(menu)
-    hide_icon = lambda: indicator.set_status (
-        AppIndicator3.IndicatorStatus.PASSIVE)
-    show_icon = lambda: indicator.set_status (
-        AppIndicator3.IndicatorStatus.ATTENTION)
-
-if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
-    import dbus.glib
-BROADCAST_INTERFACE = "org.bazaarvcs.plugins.dbus.Broadcast"
-bus = dbus.SessionBus()
-
-def catch_branch(revision_id, urls):
-    # TODO: show all the urls, or perhaps choose the 'best'.
-    url = urls[0]
-    try:
-        if isinstance(revision_id, unicode):
-            revision_id = revision_id.encode('utf8')
-        transport = get_transport(url)
-        a_dir = BzrDir.open_from_transport(transport)
-        branch = a_dir.open_branch()
-        revno = branch.revision_id_to_revno(revision_id)
-        revision = branch.repository.get_revision(revision_id)
-        summary = 'New revision %d in %s' % (revno, url)
-        body = 'Committer: %s\n' % revision.committer
-        body += 'Date: %s\n' % format_date(revision.timestamp,
-            revision.timezone)
-        body += '\n'
-        body += revision.message
-        body = cgi.escape(body)
-        nw = Notify.Notification.new(summary, body, None)
-        def start_viz(notification=None, action=None, data=None):
-            """Start the viz program."""
-            from bzrlib.plugins.gtk.commands import start_viz_window
-            pp = start_viz_window(branch, revision_id)
-            pp.show()
-        def start_branch(notification=None, action=None, data=None):
-            """Start a Branch dialog"""
-            from bzrlib.plugins.gtk.branch import BranchDialog
-            bd = BranchDialog(remote_path=url)
-            bd.run()
-        if "actions" in Notify.get_server_caps():
-            nw.add_action("inspect", "Inspect", start_viz, None)
-            nw.add_action("branch", "Branch", start_branch, None)
-        show_icon()
-        GObject.timeout_add(5000, hide_icon)
-        nw.set_timeout(5000)
-        nw.show()
-    except Exception, e:
-        print e
-        raise
-bus.add_signal_receiver(catch_branch,
-                        dbus_interface=BROADCAST_INTERFACE,
-                        signal_name="Revision")
-Notify.init("bzr-notify")
-
-if sys.argv[-1] == 'test':
-    # Exit before main loop.
-    sys.exit(0)
-
-Gtk.main()

=== removed file 'bzr-notify.desktop'
--- bzr-notify.desktop	2011-09-28 07:45:39 +0000
+++ bzr-notify.desktop	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-[Desktop Entry]
-Type=Application
-Version=0.1
-Name=Bazaar Notification
-GenericName=Bazaar Notification
-Comment=Notification Area Icon for Bazaar
-Icon=bzr-icon-64
-Exec=bzr-notify
-Categories=Application;Development;RevisionControl;
-Terminal=false
-X-GNOME-Autostart-Delay=30
-

=== removed file 'bzr-notify.desktop.in'
--- bzr-notify.desktop.in	2010-08-21 09:32:13 +0000
+++ bzr-notify.desktop.in	1970-01-01 00:00:00 +0000
@@ -1,10 +0,0 @@
-[Desktop Entry]
-Type=Application
-Version=0.1
-_Name=Bazaar Notification
-_GenericName=Bazaar Notification
-_Comment=Notification Area Icon for Bazaar
-Icon=bzr-icon-64
-Exec=bzr-notify
-Categories=Application;Development;RevisionControl;
-Terminal=false

=== modified file 'commit.py'
--- commit.py	2012-02-29 14:04:24 +0000
+++ commit.py	2012-03-22 17:14:22 +0000
@@ -30,6 +30,7 @@
     tsort,
     )
 from bzrlib.plugins.gtk.dialog import question_dialog
+from bzrlib.plugins.gtk.diff import DiffView
 from bzrlib.plugins.gtk.errors import show_bzr_error
 from bzrlib.plugins.gtk.i18n import _i18n
 from bzrlib.plugins.gtk.commitmsgs import SavedCommitMessagesManager
@@ -559,8 +560,6 @@
                                              Gtk.CellRendererText(), text=3))
 
     def _construct_diff_view(self):
-        from bzrlib.plugins.gtk.diff import DiffView
-
         # TODO: jam 2007-10-30 The diff label is currently disabled. If we
         #       decide that we really don't ever want to display it, we should
         #       actually remove it, and other references to it, along with the
@@ -575,6 +574,12 @@
         self._add_to_right_table(self._diff_view, 4, True)
         self._diff_view.show()
 
+    @staticmethod
+    def get_line_height(widget):
+        pango_layout = widget.create_pango_layout("X");
+        ink_rectangle, logical_rectangle = pango_layout.get_pixel_extents()
+        return logical_rectangle.height
+
     def _construct_file_message(self):
         scroller = Gtk.ScrolledWindow()
         scroller.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
@@ -587,6 +592,8 @@
         self._file_message_text_view.modify_font(Pango.FontDescription("Monospace"))
         self._file_message_text_view.set_wrap_mode(Gtk.WrapMode.WORD)
         self._file_message_text_view.set_accepts_tab(False)
+        line_height = self.get_line_height(self._file_message_text_view)
+        self._file_message_text_view.set_size_request(-1, line_height * 2)
         self._file_message_text_view.show()
 
         self._file_message_expander = Gtk.Expander(

=== modified file 'nautilus-bzr.py'
--- nautilus-bzr.py	2011-12-01 10:46:10 +0000
+++ nautilus-bzr.py	2012-03-05 14:02:16 +0000
@@ -124,6 +124,12 @@
         dialog.display()
         Gtk.main()
 
+    def push_cb(self, menu, controldir, path=None):
+        from bzrlib.plugins.gtk.push import PushDialog
+        dialog = PushDialog(branch=controldir.open_workingtree().branch)
+        dialog.display()
+        Gtk.main()
+
     def merge_cb(self, menu, tree, path=None):
         from bzrlib.plugins.gtk.merge import MergeDialog
         dialog = MergeDialog(tree, path)
@@ -239,6 +245,7 @@
             item.connect('activate', self.unignore_cb, tree, path)
             yield item
         else:
+            kind = tree.kind(file_id)
             item = Nautilus.MenuItem(name='BzrNautilus::log',
                              label='History ...',
                              tip='List changes',
@@ -268,12 +275,13 @@
             item.connect('activate', self.remove_cb, tree, path)
             yield item
 
-            item = Nautilus.MenuItem(name='BzrNautilus::annotate',
-                         label='Annotate ...',
-                         tip='Annotate File Data',
-                         icon='')
-            item.connect('activate', self.annotate_cb, tree, path, file_id)
-            yield item
+            if kind == 'file':
+                item = Nautilus.MenuItem(name='BzrNautilus::annotate',
+                             label='Annotate ...',
+                             tip='Annotate File Data',
+                             icon='')
+                item.connect('activate', self.annotate_cb, tree, path, file_id)
+                yield item
 
     def get_file_items(self, window, files):
         items = []

=== removed file 'notify.py'
--- notify.py	2012-02-03 18:59:38 +0000
+++ notify.py	1970-01-01 00:00:00 +0000
@@ -1,104 +0,0 @@
-# Copyright (C) 2007 by Robert Collins
-#                       Jelmer Vernooij
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-"""Notification area icon and notification for Bazaar."""
-
-from gi.repository import Gtk
-import bzrlib
-
-
-def has_dbus():
-    return (getattr(bzrlib.plugins, "dbus", None) is not None)
-
-
-def has_avahi():
-    return (getattr(bzrlib.plugins, "avahi", None) is not None)
-
-
-class NotifyPopupMenu(Gtk.Menu):
-
-    SHOW_WIDGETS = True
-
-    def __init__(self):
-        super(NotifyPopupMenu, self).__init__()
-        self.create_items()
-
-    def create_items(self):
-        from bzrlib import errors
-        item = Gtk.CheckMenuItem.new_with_mnemonic('_Gateway to LAN')
-        item.connect('toggled', self.toggle_lan_gateway)
-        self.append(item)
-        self.append(Gtk.SeparatorMenuItem())
-        try:
-            from bzrlib.plugins.dbus.activity import LanGateway
-            self.langateway = LanGateway()
-        except ImportError:
-            item.set_sensitive(False)
-        except errors.BzrError:
-            # FIXME: Should only catch errors that indicate a lan-notify
-            # process is already running.
-            item.set_sensitive(False)
-
-        item = Gtk.CheckMenuItem.new_with_mnemonic(
-            'Announce _branches on LAN')
-        item.connect('toggled', self.toggle_announce_branches)
-        self.append(item)
-        self.append(Gtk.SeparatorMenuItem())
-        try:
-            from bzrlib.plugins.avahi.share import ZeroConfServer
-            from bzrlib import urlutils
-            self.zeroconfserver = ZeroConfServer(urlutils.normalize_url('.'))
-        except ImportError:
-            item.set_sensitive(False)
-
-        item = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_PREFERENCES, None)
-        item.connect('activate', self.show_preferences)
-        self.append(item)
-        item = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_ABOUT, None)
-        item.connect('activate', self.show_about)
-        self.append(item)
-        self.append(Gtk.SeparatorMenuItem())
-        item = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_QUIT, None)
-        item.connect('activate', Gtk.main_quit)
-        self.append(item)
-        if self.SHOW_WIDGETS:
-            self.show_all()
-
-    def display(self, icon, event_button, event_time):
-        self.popup(None, None, Gtk.status_icon_position_menu,
-               event_button, event_time, icon)
-
-    def toggle_lan_gateway(self, item):
-        if item.get_active():
-            self.langateway.start()
-        else:
-            self.langateway.stop()
-
-    def toggle_announce_branches(self, item):
-        if item.get_active():
-            self.zeroconfserver.start()
-        else:
-            self.zeroconfserver.close()
-
-    def show_about(self, item):
-        from bzrlib.plugins.gtk.about import AboutDialog
-        dialog = AboutDialog()
-        dialog.run()
-
-    def show_preferences(self, item):
-        from bzrlib.plugins.gtk.preferences import PreferencesWindow
-        prefs = PreferencesWindow()
-        prefs.run()

=== modified file 'push.py'
--- push.py	2012-03-04 02:21:45 +0000
+++ push.py	2012-03-11 21:41:49 +0000
@@ -24,7 +24,8 @@
     errors,
     ui,
     )
-from bzrlib.bzrdir import BzrDir
+from bzrlib.controldir import ControlDir
+from bzrlib.push import PushResult
 from bzrlib.transport import get_transport
 
 from bzrlib.plugins.gtk.dialog import (
@@ -38,15 +39,19 @@
 class PushDialog(Gtk.Dialog):
     """New implementation of the Push dialog."""
 
-    def __init__(self, repository, revid, branch=None, parent=None):
+    def __init__(self, repository=None, revid=None, branch=None, parent=None):
         """Initialize the Push dialog. """
         super(PushDialog, self).__init__(
             title="Push", parent=parent, flags=0, border_width=6,
             buttons=(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE))
+        if repository is None:
+            repository = branch.repository
         self.branch = branch
 
         # Unused arguments
         self.repository = repository
+        if revid is None:
+            revid = branch.last_revision()
         self.revid = revid
 
         # Create the widgets
@@ -136,43 +141,62 @@
     :param overwrite: overwrite target location if it diverged
     :return: number of revisions pushed
     """
-    transport = get_transport(location)
-    location_url = transport.base
-
+    revision_id = None
+    to_transport = get_transport(location)
     try:
-        dir_to = BzrDir.open(location_url)
-        br_to = dir_to.open_branch()
+        dir_to = ControlDir.open_from_transport(to_transport)
     except errors.NotBranchError:
-        # create a branch.
-        transport = transport.clone('..')
+        dir_to = None
+
+    if dir_to is None:
         try:
-            relurl = transport.relpath(location_url)
-            transport.mkdir(relurl)
+            br_to = br_from.create_clone_on_transport(
+                to_transport, revision_id=revision_id)
         except errors.NoSuchFile:
             response = question_dialog(
                 _i18n('Non existing parent directory'),
                 _i18n("The parent directory (%s)\ndoesn't exist. Create?") %
-                location)
+                    location)
             if response == Gtk.ResponseType.OK:
-                transport.create_prefix()
+                br_to = br_from.create_clone_on_transport(
+                    to_transport, revision_id=revision_id, create_prefix=True)
             else:
-                return
-        dir_to = br_from.bzrdir.clone(location_url,
-            revision_id=br_from.last_revision())
-        br_to = dir_to.open_branch()
-        count = len(br_to.revision_history())
+                return _i18n("Push aborted.")
+        push_result = create_push_result(br_from, br_to)
     else:
-        br_to.revision_history()
-        try:
-            tree_to = dir_to.open_workingtree()
-        except errors.NotLocalUrl:
-            # FIXME - what to do here? how should we warn the user?
-            count = br_to.pull(br_from, overwrite)
-        except errors.NoWorkingTree:
-            count = br_to.pull(br_from, overwrite)
-        else:
-            count = tree_to.pull(br_from, overwrite)
-
-    # The count var is either an int or a PushResult. PushResult is being
-    # coerced into an int, but the method is deprecated.
-    return _i18n("%d revision(s) pushed.") % int(count)
+        push_result = dir_to.push_branch(br_from, revision_id, overwrite)
+    message = create_push_message(br_from, push_result)
+    return message
+
+
+def create_push_message(br_from, push_result):
+    """Return a mesage explaining what happened during the push."""
+    messages = []
+    rev_count = br_from.revno() - push_result.old_revno
+    messages.append(_i18n("%d revision(s) pushed.") % rev_count)
+    if push_result.stacked_on is not None:
+        messages.append(_i18n("Stacked on %s.") % push_result.stacked_on)
+    if push_result.workingtree_updated is False:
+        messages.append(_i18n(
+            "\nThe working tree was not updated:"
+            "\nSee 'bzr help working-trees' for more information."))
+    return '\n'.join(messages)
+
+
+def create_push_result(br_from, br_to):
+    """Return a PushResult like one created by ControlDir.push_branch()."""
+    push_result = PushResult()
+    push_result.source_branch = br_from
+    push_result.target_branch = br_to
+    push_result.branch_push_result = None
+    push_result.master_branch = None
+    push_result.old_revno = 0
+    push_result.old_revid = br_to.last_revision()
+    push_result.workingtree_updated = None  # Not applicable to this case.
+    try:
+        push_result.stacked_on = br_to.get_stacked_on_url()
+    except (errors.UnstackableBranchFormat,
+            errors.UnstackableRepositoryFormat,
+            errors.NotStacked):
+        push_result.stacked_on = None
+    return push_result

=== modified file 'setup.py'
--- setup.py	2012-02-28 17:45:56 +0000
+++ setup.py	2012-07-09 15:23:26 +0000
@@ -148,7 +148,7 @@
         maintainer_email="jelmer@samba.org",
         description="GTK+ Frontends for various Bazaar commands",
         license="GNU GPL v2 or later",
-        scripts=['bzr-handle-patch', 'bzr-notify'],
+        scripts=['bzr-handle-patch'],
         url="http://bazaar-vcs.org/BzrGtk",
         package_dir={
             "bzrlib.plugins.gtk": ".",
@@ -188,8 +188,7 @@
                     # In case Python distutils extra is not available,
                     # install the .desktop files
                     ('share/applications', ['bazaar-properties.desktop',
-                                            'bzr-handle-patch.desktop',
-                                            'bzr-notify.desktop']),
+                                            'bzr-handle-patch.desktop']),
                     ('share/application-registry', ['bzr-gtk.applications']),
                     ('share/pixmaps', ['icons/bzr-icon-64.png']),
                     ('share/icons/hicolor/scalable/apps',

=== modified file 'tests/test_commit.py'
--- tests/test_commit.py	2012-02-17 15:03:17 +0000
+++ tests/test_commit.py	2012-03-22 17:14:22 +0000
@@ -235,6 +235,13 @@
         dlg._on_treeview_files_cursor_changed(treeview)
         self.assertFalse(CommitDialogNoWidgets._update_per_file_info.called)
 
+    def test_get_line_height(self):
+        tree = self.make_branch_and_tree('tree')
+        dlg = CommitDialogNoWidgets(tree)
+        textview = Gtk.TextView()
+        line_height = dlg.get_line_height(textview)
+        self.assertIsInstance(line_height, int)
+
 
 class TestCommitDialog(tests.TestCaseWithTransport):
 

=== modified file 'tests/test_push.py'
--- tests/test_push.py	2012-03-04 01:54:22 +0000
+++ tests/test_push.py	2012-03-20 12:39:08 +0000
@@ -45,8 +45,8 @@
         dialog = push.PushDialog(
             repository=None, revid=None, branch=branch, parent=None)
         self.assertIs(None, dialog.props.parent)
-        self.assertIs(None, dialog.repository)
-        self.assertIs(None, dialog.revid)
+        self.assertIs(branch.repository, dialog.repository)
+        self.assertIs(branch.last_revision(), dialog.revid)
         self.assertIs(branch, dialog.branch)
         # Layout rules to match HIG.
         self.assertIsInstance(dialog._label_location, Gtk.Label)
@@ -133,3 +133,98 @@
             (branch, 'lp:~user/fnord/test'), push.do_push.args)
         self.assertEqual(
             {'overwrite': True}, push.do_push.kwargs)
+
+
+class DoPushTestCase(tests.TestCaseWithTransport):
+
+    def setup_ui(self):
+        set_ui_factory()
+        progress_panel = ProgressPanel()
+        ui.ui_factory.set_progress_bar_widget(progress_panel)
+        MockMethod.bind(self, progress_panel.pb, 'tick')
+        MockMethod.bind(self, progress_panel.pb, 'update')
+        MockMethod.bind(self, progress_panel.pb, 'finished')
+        return progress_panel
+
+    def make_from_branch(self):
+        from_tree = self.make_branch_and_tree('this')
+        self.build_tree(['this/a', 'this/b'])
+        from_tree.add(['a', 'b'])
+        from_tree.commit("msg")
+        return from_tree.branch
+
+    def test_do_push_without_dir(self):
+        progress_panel = self.setup_ui()
+        from_branch = self.make_from_branch()
+        message = push.do_push(from_branch, 'that', False)
+        self.assertEqual('1 revision(s) pushed.', message)
+        self.assertEqual(True, progress_panel.pb.update.called)
+        self.assertEqual(True, progress_panel.pb.finished.called)
+
+    def test_do_push_without_parent_dir(self):
+        progress_panel = self.setup_ui()
+        from_branch = self.make_from_branch()
+        MockMethod.bind(self, push, 'question_dialog', Gtk.ResponseType.OK)
+        message = push.do_push(from_branch, 'that/there', False)
+        self.assertEqual('1 revision(s) pushed.', message)
+        self.assertEqual(True, push.question_dialog.called)
+        self.assertEqual(True, progress_panel.pb.update.called)
+        self.assertEqual(True, progress_panel.pb.finished.called)
+
+    def test_do_push_without_parent_dir_aborted(self):
+        self.setup_ui()
+        from_branch = self.make_from_branch()
+        MockMethod.bind(self, push, 'question_dialog', Gtk.ResponseType.CANCEL)
+        message = push.do_push(from_branch, 'that/there', False)
+        self.assertEqual('Push aborted.', message)
+
+    def test_do_push_with_dir(self):
+        progress_panel = self.setup_ui()
+        self.make_branch_and_tree('that')
+        from_branch = self.make_from_branch()
+        message = push.do_push(from_branch, 'that', False)
+        self.assertEqual('1 revision(s) pushed.', message)
+        self.assertEqual(True, progress_panel.pb.update.called)
+        self.assertEqual(True, progress_panel.pb.finished.called)
+
+    def test_create_push_result_unstacked(self):
+        from_branch = object()
+        to_branch = self.make_branch_and_tree('that').branch
+        push_result = push.create_push_result(from_branch, to_branch)
+        self.assertIs(from_branch, push_result.source_branch)
+        self.assertIs(to_branch, push_result.target_branch)
+        self.assertIs(None, push_result.branch_push_result)
+        self.assertIs(None, push_result.master_branch)
+        self.assertEqual(0, push_result.old_revno)
+        self.assertEqual(to_branch.last_revision(), push_result.old_revid)
+        self.assertIs(None, push_result.workingtree_updated)
+        self.assertIs(None, push_result.stacked_on)
+
+    def test_create_push_result_stacked(self):
+        from_branch = object()
+        to_branch = self.make_branch_and_tree('that').branch
+        MockMethod.bind(self, to_branch, 'get_stacked_on_url', 'lp:project')
+        push_result = push.create_push_result(from_branch, to_branch)
+        self.assertEqual('lp:project', push_result.stacked_on)
+
+    def test_create_push_message_stacked_on(self):
+        from_branch = object()
+        to_branch = self.make_branch_and_tree('that').branch
+        MockMethod.bind(self, to_branch, 'get_stacked_on_url', 'lp:project')
+        push_result = push.create_push_result(from_branch, to_branch)
+        from_branch = self.make_from_branch()
+        message = push.create_push_message(from_branch, push_result)
+        self.assertEqual(
+            '1 revision(s) pushed.\nStacked on lp:project.', message)
+
+    def test_create_push_message_workingtree_updated_false(self):
+        from_branch = object()
+        to_branch = self.make_branch_and_tree('that').branch
+        push_result = push.create_push_result(from_branch, to_branch)
+        push_result.workingtree_updated = False
+        from_branch = self.make_from_branch()
+        message = push.create_push_message(from_branch, push_result)
+        self.assertEqual(
+            "1 revision(s) pushed.\n\nThe working tree was not updated:\n"
+            "See 'bzr help working-trees' for more information.",
+            message)

